@@ -20,7 +20,6 @@ from oslo_log import log as logging
 from oslo_utils import strutils
 from oslo_utils import uuidutils
 import pecan
-import six
 
 from zun.api.controllers import base
 from zun.api.controllers import link
@@ -92,7 +91,7 @@ class ContainersActionsController(base.Controller):
         context = pecan.request.context
         policy.enforce(context, "container:actions",
                        action="container:actions")
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         actions_raw = objects.ContainerAction.get_by_container_uuid(
             context, container.uuid)
         actions = [actions_view.format_action(a) for a in actions_raw]
@@ -107,7 +106,7 @@ class ContainersActionsController(base.Controller):
         context = pecan.request.context
         policy.enforce(context, "container:actions",
                        action="container:actions")
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         action = objects.ContainerAction.get_by_request_id(
             context, container.uuid, request_ident)
 
@@ -228,7 +227,7 @@ class ContainersController(base.Controller):
             policy.enforce(context, "container:get_one_all_projects",
                            action="container:get_one_all_projects")
             context.all_projects = True
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:get_one")
         if container.host:
             compute_api = pecan.request.compute_api
@@ -258,7 +257,7 @@ class ContainersController(base.Controller):
         # '"nginx" "-g" "daemon off;"' -> ["nginx", "-g", "daemon off;"]
         command = container_dict.pop('command', None)
         if command is not None:
-            if isinstance(command, six.string_types):
+            if isinstance(command, str):
                 command = shlex.split(command)
             container_dict['command'] = command
 
@@ -270,19 +269,37 @@ class ContainersController(base.Controller):
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_create)
     @validation.validated(schema.container_create)
-    def post(self, run=False, **container_dict):
+    def post(self, run=False, **container_dict):  # noqa
         interactive = container_dict.get('interactive')
         if interactive is not None:
             container_dict['tty'] = interactive
         return self._do_post(run, **container_dict)
 
-    @base.Controller.api_version("1.36")  # noqa
+    @base.Controller.api_version("1.36", "1.38")  # noqa
     @pecan.expose('json')
     @api_utils.enforce_content_types(['application/json'])
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_create)
     @validation.validated(schema.container_create)
-    def post(self, run=False, **container_dict):
+    def post(self, run=False, **container_dict):  # noqa
+        return self._do_post(run, **container_dict)
+
+    @base.Controller.api_version("1.39", "1.39")  # noqa
+    @pecan.expose('json')
+    @api_utils.enforce_content_types(['application/json'])
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_create)
+    @validation.validated(schema.container_create_v139)
+    def post(self, run=False, **container_dict):  # noqa
+        return self._do_post(run, **container_dict)
+
+    @base.Controller.api_version("1.40")  # noqa
+    @pecan.expose('json')
+    @api_utils.enforce_content_types(['application/json'])
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_create)
+    @validation.validated(schema.container_create_v140)
+    def post(self, run=False, **container_dict):  # noqa
         return self._do_post(run, **container_dict)
 
     def _do_post(self, run=False, **container_dict):
@@ -403,12 +420,18 @@ class ContainersController(base.Controller):
             registry = utils.get_registry(registry)
             container_dict['registry_id'] = registry.id
 
+        requested_host = container_dict.pop('host', None)
+        if requested_host:
+            policy.enforce(context, "container:create:requested_destination",
+                           action="container:create:requested_destination")
+
         container_dict['status'] = consts.CREATING
         extra_spec = {}
         extra_spec['hints'] = container_dict.get('hints', None)
         extra_spec['pci_requests'] = pci_req
         extra_spec['availability_zone'] = container_dict.get(
             'availability_zone')
+        extra_spec['requested_host'] = requested_host
         new_container = objects.Container(context, **container_dict)
         new_container.create(context)
 
@@ -457,7 +480,7 @@ class ContainersController(base.Controller):
                                                        project_id)
                 total = None
                 try:
-                    if isinstance(count_as_dict[res_name], six.integer_types):
+                    if isinstance(count_as_dict[res_name], int):
                         total = count_as_dict[res_name] + int(res_delta)
                     else:
                         total = float(count_as_dict[res_name]) + \
@@ -558,7 +581,6 @@ class ContainersController(base.Controller):
         for mount in mounts:
             volume_dict = {
                 'cinder_volume_id': None,
-                'container_path': None,
                 'auto_remove': False,
                 'contents': None,
                 'user_id': context.user_id,
@@ -570,20 +592,22 @@ class ContainersController(base.Controller):
                     volume = cinder_api.search_volume(mount['source'])
                     cinder_api.ensure_volume_usable(volume)
                     volume_dict['cinder_volume_id'] = volume.id
-                    volume_dict['container_path'] = mount['destination']
                     volume_dict['volume_provider'] = 'cinder'
                 elif mount.get('size'):
                     volume = cinder_api.create_volume(mount['size'])
                     cinder_api.ensure_volume_usable(volume)
                     volume_dict['cinder_volume_id'] = volume.id
-                    volume_dict['container_path'] = mount['destination']
                     volume_dict['volume_provider'] = 'cinder'
                     volume_dict['auto_remove'] = True
             elif volume_type == 'bind':
                 volume_dict['contents'] = mount.pop('source', '')
-                volume_dict['container_path'] = mount['destination']
                 volume_dict['volume_provider'] = 'local'
 
+            volume_object = objects.Volume(context, **volume_dict)
+            volume_object.create(context)
+            volume_dict['volume_id'] = volume_object.id
+            volume_dict['container_path'] = mount['destination']
+
             volmapp = objects.VolumeMapping(context, **volume_dict)
             requested_volumes[container.uuid].append(volmapp)
 
@@ -594,7 +618,7 @@ class ContainersController(base.Controller):
         try:
             return neutron_api.find_resourceid_by_name_or_id(
                 'security_group', security_group['name'], context.project_id)
-        except n_exc.NeutronClientNoUniqueMatch as e:
+        except n_exc.NeutronClientNoUniqueMatch:
             msg = _("Multiple security group matches found for name "
                     "%(name)s, use an ID to be more specific.") % {
                 'name': security_group['name']}
@@ -618,7 +642,7 @@ class ContainersController(base.Controller):
         :param security_group: security_group to be added to container.
         """
 
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(
             container.as_dict(), "container:add_security_group")
         utils.validate_container_state(container, 'add_security_group')
@@ -645,7 +669,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or Name of a container.
         :param security_group: security_group to be removed from container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(
             container.as_dict(), "container:remove_security_group")
         utils.validate_container_state(container, 'remove_security_group')
@@ -670,7 +694,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or name of a container.
         :param patch: a json PATCH document to apply to this container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         context = pecan.request.context
         container_deltas = {}
         check_policy_on_container(container.as_dict(), "container:update")
@@ -709,7 +733,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or Name of a container.
         :param name: a new name for this container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:rename")
         if container.name == name:
             raise exception.Conflict('The new name for the container is the '
@@ -730,7 +754,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or name of a container.
         :param kwargs: cpu/memory to be updated.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(),
                                   "container:resize_container")
         utils.validate_container_state(container, 'resize_container')
@@ -759,7 +783,7 @@ class ContainersController(base.Controller):
             policy.enforce(context, "container:delete_all_projects",
                            action="container:delete_all_projects")
             context.all_projects = True
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:delete")
         stop = kwargs.pop('stop', False)
         try:
@@ -801,7 +825,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:rebuild")
         utils.validate_container_state(container, 'rebuild')
         if kwargs.get('image'):
@@ -826,7 +850,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:start")
         utils.validate_container_state(container, 'start')
         LOG.debug('Calling compute.container_start with %s',
@@ -844,7 +868,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:stop")
         utils.validate_container_state(container, 'stop')
         LOG.debug('Calling compute.container_stop with %s',
@@ -862,7 +886,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:reboot")
         utils.validate_container_state(container, 'reboot')
         LOG.debug('Calling compute.container_reboot with %s',
@@ -881,7 +905,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:pause")
         utils.validate_container_state(container, 'pause')
         LOG.debug('Calling compute.container_pause with %s',
@@ -898,7 +922,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:unpause")
         utils.validate_container_state(container, 'unpause')
         LOG.debug('Calling compute.container_unpause with %s',
@@ -924,7 +948,7 @@ class ContainersController(base.Controller):
         :param since: Show logs since a given datetime or
                      integer epoch (in seconds).
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:logs")
         utils.validate_container_state(container, 'logs')
         try:
@@ -954,7 +978,7 @@ class ContainersController(base.Controller):
         :param interactive: Keep STDIN open and allocate a
                             pseudo-TTY for interactive.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:execute")
         utils.validate_container_state(container, 'execute')
         try:
@@ -983,7 +1007,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or Name of a container.
         :param exec_id: ID of a exec.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(),
                                   "container:execute_resize")
         utils.validate_container_state(container, 'execute_resize')
@@ -1002,7 +1026,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:kill")
         utils.validate_container_state(container, 'kill')
         LOG.debug('Calling compute.container_kill with %(uuid)s '
@@ -1021,7 +1045,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:attach")
         utils.validate_container_state(container, 'attach')
         LOG.debug('Checking the status for attach with %s', container.uuid)
@@ -1042,7 +1066,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:resize")
         utils.validate_container_state(container, 'resize')
         LOG.debug('Calling tty resize with %s ', container.uuid)
@@ -1060,7 +1084,7 @@ class ContainersController(base.Controller):
         :param container_ident: UUID or Name of a container.
         :param ps_args: The args of the ps command.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:top")
         utils.validate_container_state(container, 'top')
         LOG.debug('Calling compute.container_top with %s', container.uuid)
@@ -1084,7 +1108,7 @@ class ContainersController(base.Controller):
     @base.Controller.api_version("1.25")  # noqa
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def get_archive(self, container_ident, **kwargs):
+    def get_archive(self, container_ident, **kwargs):  # noqa
         """Retrieve a file/folder from a container
 
         Retrieve a file or folder from a container in the
@@ -1095,7 +1119,7 @@ class ContainersController(base.Controller):
         return self._get_archive(container_ident, **kwargs)
 
     def _get_archive(self, container_ident, **kwargs):
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:get_archive")
         utils.validate_container_state(container, 'get_archive')
         LOG.debug('Calling compute.container_get_archive with %(uuid)s '
@@ -1123,7 +1147,7 @@ class ContainersController(base.Controller):
     @base.Controller.api_version("1.25")  # noqa
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def put_archive(self, container_ident, **kwargs):
+    def put_archive(self, container_ident, **kwargs):  # noqa
         """Insert a file/folder to container.
 
         Insert a file or folder to an existing container using
@@ -1134,7 +1158,7 @@ class ContainersController(base.Controller):
         self._put_archive(container_ident, **kwargs)
 
     def _put_archive(self, container_ident, **kwargs):
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:put_archive")
         utils.validate_container_state(container, 'put_archive')
         LOG.debug('Calling compute.container_put_archive with %(uuid)s '
@@ -1153,7 +1177,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:stats")
         utils.validate_container_state(container, 'stats')
         LOG.debug('Calling compute.container_stats with %s', container.uuid)
@@ -1169,7 +1193,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(), "container:commit")
         utils.validate_container_state(container, 'commit')
         LOG.debug('Calling compute.container_commit %s ', container.uuid)
@@ -1189,7 +1213,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(),
                                   "container:network_detach")
         context = pecan.request.context
@@ -1228,7 +1252,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         check_policy_on_container(container.as_dict(),
                                   "container:network_attach")
         context = pecan.request.context
@@ -1258,7 +1282,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         container_networks = self._get_container_networks_legacy(container)
         return {'networks': container_networks}
 
@@ -1278,12 +1302,12 @@ class ContainersController(base.Controller):
     @base.Controller.api_version("1.18")  # noqa
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def network_list(self, container_ident):
+    def network_list(self, container_ident):  # noqa
         """Retrieve a list of networks of the container.
 
         :param container_ident: UUID or Name of a container.
         """
-        container = utils.get_container(container_ident)
+        container = api_utils.get_resource('Container', container_ident)
         container_networks = self._get_container_networks(container)
         return {'networks': container_networks}
 
