@@ -40,6 +40,7 @@ _basic_keys = (
     'restart_policy',
     'status_detail',
     'interactive',
+    'tty',
     'image_driver',
     'security_groups',
     'auto_remove',
@@ -50,6 +51,9 @@ _basic_keys = (
     'healthcheck',
     'cpu_policy',
     'registry_id',
+    'entrypoint',
+    'created_at',
+    'updated_at',
 )
 
 
