@@ -239,7 +239,7 @@ class Connection(object):
             query = add_identity_filter(query, container_id)
             count = query.delete()
             if count != 1:
-                raise exception.ContainerNotFound(container_id)
+                raise exception.ContainerNotFound(container=container_id)
 
     def update_container(self, context, container_type, container_id, values):
         # NOTE(dtantsur): this can lead to very strange errors
@@ -259,7 +259,7 @@ class Connection(object):
             query = self._add_container_type_filter(container_type, query)
             query = add_identity_filter(query, container_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ContainerNotFound(container=container_id)
 
@@ -309,7 +309,8 @@ class Connection(object):
         try:
             return query.one()
         except NoResultFound:
-            raise exception.VolumeMappingNotFound(volume_mapping_uuid)
+            raise exception.VolumeMappingNotFound(
+                volume_mapping=volume_mapping_uuid)
 
     def destroy_volume_mapping(self, context, volume_mapping_uuid):
         session = get_session()
@@ -335,9 +336,10 @@ class Connection(object):
             query = model_query(models.VolumeMapping, session=session)
             query = add_identity_filter(query, volume_mapping_uuid)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
-                raise exception.VolumeMappingNotFound(volume_mapping_uuid)
+                raise exception.VolumeMappingNotFound(
+                    volume_mapping=volume_mapping_uuid)
 
             ref.update(values)
         return ref
@@ -369,7 +371,7 @@ class Connection(object):
         try:
             return query.one()
         except NoResultFound:
-            raise exception.VolumeNotFound(volume_id)
+            raise exception.VolumeNotFound(volume=volume_id)
 
     def destroy_volume(self, context, volume_uuid):
         session = get_session()
@@ -378,7 +380,7 @@ class Connection(object):
             query = add_identity_filter(query, volume_uuid)
             count = query.delete()
             if count != 1:
-                raise exception.VolumeNotFound(volume_uuid)
+                raise exception.VolumeNotFound(volume=volume_uuid)
 
     def update_volume(self, context, volume_uuid, values):
         # NOTE(dtantsur): this can lead to very strange errors
@@ -394,9 +396,9 @@ class Connection(object):
             query = model_query(models.Volume, session=session)
             query = add_identity_filter(query, volume_uuid)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
-                raise exception.VolumeNotFound(volume_uuid)
+                raise exception.VolumeNotFound(volume=volume_uuid)
 
             ref.update(values)
         return ref
@@ -416,7 +418,7 @@ class Connection(object):
             query = model_query(models.ZunService, session=session)
             query = query.filter_by(host=host, binary=binary)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ZunServiceNotFound(host=host, binary=binary)
 
@@ -471,7 +473,7 @@ class Connection(object):
             query = add_identity_filter(query, uuid)
             count = query.delete()
             if count != 1:
-                raise exception.ImageNotFound(uuid)
+                raise exception.ImageNotFound(image=uuid)
 
     def pull_image(self, context, values):
         # ensure defaults are present for new images
@@ -499,7 +501,7 @@ class Connection(object):
             query = model_query(models.Image, session=session)
             query = add_identity_filter(query, image_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ImageNotFound(image=image_id)
 
@@ -614,7 +616,7 @@ class Connection(object):
             query = model_query(models.ResourceProvider, session=session)
             query = add_identity_filter(query, provider_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ResourceProviderNotFound(
                     resource_provider=provider_id)
@@ -676,7 +678,7 @@ class Connection(object):
             query = model_query(models.ResourceClass, session=session)
             query = query.filter_by(id=resource_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ResourceClassNotFound(
                     resource_class=resource_id)
@@ -738,7 +740,7 @@ class Connection(object):
             query = model_query(models.Inventory, session=session)
             query = query.filter_by(id=inventory_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.InventoryNotFound(inventory=inventory_id)
 
@@ -797,7 +799,7 @@ class Connection(object):
             query = model_query(models.Allocation, session=session)
             query = query.filter_by(id=allocation_id)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.AllocationNotFound(allocation=allocation_id)
 
@@ -805,7 +807,7 @@ class Connection(object):
         return ref
 
     def _add_compute_nodes_filters(self, query, filters):
-        filter_names = ['hostname']
+        filter_names = ['hostname', 'rp_uuid']
         return self._add_filters(query, models.ComputeNode, filters=filters,
                                  filter_names=filter_names)
 
@@ -821,6 +823,8 @@ class Connection(object):
         # ensure defaults are present for new compute nodes
         if not values.get('uuid'):
             values['uuid'] = uuidutils.generate_uuid()
+        if not values.get('rp_uuid'):
+            values['rp_uuid'] = values['uuid']
 
         compute_node = models.ComputeNode()
         compute_node.update(values)
@@ -875,7 +879,7 @@ class Connection(object):
             query = model_query(models.ComputeNode, session=session)
             query = query.filter_by(uuid=node_uuid)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.ComputeNodeNotFound(
                     compute_node=node_uuid)
@@ -945,6 +949,17 @@ class Connection(object):
         action.save()
         return action
 
+    def action_finish(self, context, values):
+        query = model_query(models.ContainerAction).\
+            filter_by(container_uuid=values['container_uuid']).\
+            filter_by(request_id=values['request_id']).\
+            filter_by(action=values['action'])
+        if query.update(values) != 1:
+            raise exception.ContainerActionNotFound(
+                request_id=values['request_id'],
+                container_uuid=values['container_uuid'])
+        return query.one()
+
     def actions_get(self, context, container_uuid):
         """Get all container actions for the provided uuid."""
         query = model_query(models.ContainerAction).\
@@ -1020,6 +1035,7 @@ class Connection(object):
             raise exception.ContainerActionNotFound(
                 request_id=values['request_id'],
                 container_uuid=values['container_uuid'])
+
         event = model_query(models.ContainerActionEvent).\
             filter_by(action_id=action['id']).\
             filter_by(event=values['event']).\
@@ -1032,10 +1048,6 @@ class Connection(object):
         event.update(values)
         event.save()
 
-        if values['result'].lower() == 'error':
-            action.update({'message': 'Error'})
-            action.save()
-
         return event
 
     def action_events_get(self, context, action_id):
@@ -1226,7 +1238,7 @@ class Connection(object):
             query = model_query(models.Network, session=session)
             query = add_identity_filter(query, network_uuid)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.NetworkNotFound(network=network_uuid)
 
@@ -1352,7 +1364,7 @@ class Connection(object):
             query = model_query(models.Registry, session=session)
             query = add_identity_filter(query, registry_uuid)
             try:
-                ref = query.with_lockmode('update').one()
+                ref = query.with_for_update().one()
             except NoResultFound:
                 raise exception.RegistryNotFound(registry=registry_uuid)
 
