@@ -26,7 +26,7 @@ import time
 import docker
 from oslo_log import log as logging
 from oslo_utils import uuidutils
-import six.moves.urllib.parse as urlparse
+from urllib import parse as urlparse
 import websockify
 
 from zun.common import context
@@ -120,6 +120,8 @@ class ZunProxyRequestHandlerBase(object):
                     'host': self.server.target_host,
                     'port': self.server.target_port})
                 raise self.CClose(1000, "Target closed")
+            if isinstance(buf, str):
+                buf = buf.encode()
             self.cqueue.append(buf)
 
     def do_websocket_proxy(self, target):
